"""
Contents producing plots: statistical and tier ones

This file is  HTML-free.
TO DO:
""" 

from time import *
from SeriesLib import *
from datetime import datetime
from PlotLib import *
from TableContents import Content
import Utils,PlotLib,SeriesLib,Forms,Tables
from Parameters import *



class PlotContent(Content):
  """
  Generic Content for plots
  """
  def getSummary(self):
    """
    This method must be called after the query is executed (startTime and endTime are uesed!)
    """
    summary=""
    for p in self.parameters:
      ### Date PArameter is special *might* need to update its values
      if isinstance(p,DateParameter):
        try: 
          p.setInterval(datetime.fromtimestamp(self.startTime),datetime.fromtimestamp(self.endTime))
        except:
          pass
      summary+=p.printValue()
    summary+="<BR/>"+Forms.PlotForm(self)
    return Tables.plotPanel(summary,self.name)

###########################################################################################################################################################################3
class StatPlotContent(PlotContent):
  """ Statistical plots are Bar Plots
   POSSIBLE INPUTS: step, hours, starting date (hour/day/month/year) and run_id + autotime option (shows the most important area of the plot automatically)
   """
  # parameters ORDER IS IMPORTANT for the Form/summary building
  parameters=[RunParameter(),HourParameter(),StepParameter(),DateParameter(),AutotimeParameter()]
  dataDecorator='simplePlot'
              
  def buildUrls(self):
    #startTime and endTime are compulsory values [I use them in several places]
    self.startTime=""
    self.endTime=""
    if self.parameterDict['autotime']==False or (self.parameterDict['autotime']==True and self.parameterDict['run_id']==None):
      #if autotime option is NOT enabled I set the start/endTime according with the parameters (otehrwise they must be left empty)
      self.startTime=int(mktime(self.parameterDict['etime'].timetuple()))-3600*self.parameterDict['hours']
      self.endTime=int(mktime(self.parameterDict['etime'].timetuple()))
      
    DASoptions='?'+'stime='+str(self.startTime)+'&amp;etime='+str(self.endTime)
    ##runId
    DASoptions+='&amp;'+self.parameters[0].getDasReq()
    # there is one case where url are multiple: I need to add the options to all the urls
    if type(self.url)==StringType:
      self.url=self.url+DASoptions
      return [self.url]
    else:
      newUrl=[]
      for u in self.url:
        newUrl.append(u+DASoptions)
      self.url=newUrl
      return newUrl
      
  def dataProcessing(self):
    plotData=[]
    if type(self.url)==StringType:
      plotData=self.resDict[self.url]
    else:
      if type(self.url)==ListType:
          for url in self.url:
            plotData+=self.resDict[url]
    if (self.parameterDict['autotime']==True):
      ##I need to set start/endTime which are not set if autotime option was enabled
      ## if there is not data I can't have min and max: anyway the plot would fail later with this exception
      if plotData==None or len(plotData)==0: 
        raise EmptyPlotException()
      self.startTime=plotData[min(range(0,len(plotData)),key=lambda w: plotData[w]['starttime'])]['starttime']
      self.endTime=  plotData[max(range(0,len(plotData)),key=lambda w: plotData[w]['endtime'])]['endtime'] ## get max value in endtime
    ## make an interval series for the plot
    series=IntervalSeries(self.name,int(self.startTime),int(self.endTime),self.parameterDict['step']*60) #minutes to seconds
    ## fill series with the data
    for line in plotData:
        series.addInfo(line)
    ##create plot
    plotClass=getattr(PlotLib,self.plotName)
    plot=plotClass(self.config,series)
    ##save plot image 
    return plot.getPlotImg()

              
        
#######################################################################################################################################
class ExpressStat(StatPlotContent):
  url='expresslifetimes'
  plotName='ExpressBarPlot'
  
class ExpressMergeStat(StatPlotContent):
  url='expressmergelifetimes'
  plotName='ExpressMergeBarPlot'

class RepackStat(StatPlotContent):
  url='repacklifetimes'
  plotName='RepackBarPlot'

class RepackMergeStat(StatPlotContent):
  url='repackmergelifetimes'
  plotName='RepackMergeBarPlot'
  
class RecoStat(StatPlotContent):
  url='recolifetimes'
  plotName='RecoBarPlot'
  
class RecoMergeStat(StatPlotContent):
  url='recomergelifetimes'
  plotName='RecoMergeBarPlot'
  
class AlcaStat(StatPlotContent):
  url='alcalifetimes'
  plotName='AlcaBarPlot'
  
class AlcaMergeStat(StatPlotContent):
  url='alcamergelifetimes'
  plotName='AlcaMergeBarPlot'
  
class AllJobsStat(StatPlotContent):
  plotName='AllJobsBarPlot'
  parameters=[RunParameter(),HourParameter((1,200)),StepParameter((1,60)),DateParameter(),AutotimeParameter()]
  url=['expresslifetimes','expressmergelifetimes',
       'repacklifetimes','repackmergelifetimes',
       'recolifetimes','recomergelifetimes',
        'alcalifetimes','alcamergelifetimes']
################################################################################################################################################
class TierPlotsContent(PlotContent):
  """
   tier+ streamers Plot content
  """
  dataDecorator='allPlots'
  ## parameters with their ranges
  parameters=[TierParameter(),HourParameter((1,8),6),StepParameter((0,60)),DateParameter()]
  layerCont=['events','eventsizes','datarate']
       
  def buildUrls(self):
      self.queryTime=self.parameterDict['hours']*60*60
      # round at +/- 5 minutes (to acutally use caching)
      self.endTime=math.floor(mktime(self.parameterDict['etime'].timetuple())/300)*300
      self.startTime=self.endTime-self.queryTime
      DASoptions='?'+'stime='+str(int(self.startTime))+'&amp;etime='+str(int(self.endTime))+'&amp;tier='+self.parameterDict['tier']
      self.url=self.url+DASoptions
      return [self.url,]
      
  def dataProcessing(self):
    """
    Tier plot is actually a sets of 3 plots
    """
    plotData=self.resDict[self.url]
    res=[]
    sets=self.getSets(plotData,self.seriesField)
    legenda=Legend(sets)
    # create series dividing data in 'dataset's
    eventSeries=self.splitInfoInSets(plotData,'DatasetSeries',self.seriesField)
    plots={self.layerCont[0]:EventsPlot(self.config,eventSeries,legenda),
           self.layerCont[1]:EventSizesPlot(self.config,eventSeries,legenda),
           self.layerCont[2]:DataRatePlot(self.config,eventSeries,legenda)}
           
    if self.parameterDict['step']==0:
      # if there is no interval I need to use  the NoInt[erval] plot objects
      plots={self.layerCont[0]:EventsPlotNoInt(self.config,eventSeries,legenda),
             self.layerCont[1]:EventSizesPlotNoInt(self.config,eventSeries,legenda),
             self.layerCont[2]:DataRatePlot(self.config,eventSeries,legenda)} 
    #make the plots and save their results
    for i in range(0,len(self.layerCont)):
      p=plots[self.layerCont[i]]
      p.makeTitle(self.parameterDict['hours'],self.parameterDict['step'])
      try:
       # I ran into a strange case: in a execution all the data where set at the same second
       # DataRate in this case was failing, but the other plots were ok [even though a little strange]
       res.append(p.getPlotImg())
      except EmptyPlotException,e:
        print e
    if res==[]: raise EmptyPlotException() # no plot has been successfully produced :(
    return res
   
 
  def getSets(self,inlist,name):
    """
    get set of streams/dataserts
    """
    sets=set()
    sets.update(map(lambda dict :dict[name], inlist))
    return sets
   
  def splitInfoInSets(self,inList,serieClass,serieNameAttr):
    """
    This function create 1 seriesClass for every set with the same 'serieNameAttr'.
    It is also possible to use a filtering function for the lines to be added to the seire.
    """
    step=self.parameterDict['step']
    if step<1: step=1
    dataSeries={}
    for line in inList:
      try:
        #print "line attr="+str(line[serieNameAttr])
        datasetSerie=dataSeries[line[serieNameAttr]]
      except:
        serie=getattr(SeriesLib,serieClass)
        datasetSerie=serie(line[serieNameAttr],step*60)
        dataSeries.update({line[serieNameAttr]: datasetSerie})
        #I use a filterFunction to divide the datasets
      if self.filterFunction(line):
          datasetSerie.addInfo(line)
      #else: print "filter failed! on line="+str(line)
    return dataSeries
 
   
  def filterFunction(self,x):
    return True     
              
             
 ###########################################################################
class StreamersTierPlots(TierPlotsContent):
  url="laststreamssummary"
  layerCont=['StreamEvents','StreamEventsizes','StreamDatarate']
  seriesField='stream'
  
  def filterFunction(self,x):
    return True
  
###################################################################################################################### 
class AllStreamPlots(TierPlotsContent):
  url="lastfilessummary"  
  layerCont=['Events','Eventsizes','Datarate']
  seriesField='dataset'
  def filterFunction(self,x):
    return True
  
###############################3
class AStreamTierPlots(AllStreamPlots):
  def filterFunction(self,x):
   return  x['stream']=='A' 
  
class OtherStreamTierPlots(AllStreamPlots):
  def filterFunction(self,x):
    return x['stream']=='others'

class ExpressStreamTierPlots(AllStreamPlots):
   def filterFunction(self,x):
     return  x['stream']=='Express'
