"""
_T0Mon request handler_
 This class sorts out the different T0Mon requests
"""

from WMCore.WebTools.Page import TemplatedPage
from cherrypy import expose
import cherrypy, TableContents, os,Utils,PlotContents,Forms
from cherrypy.lib.static import serve_file

from datetime import datetime, timedelta
from threading import BoundedSemaphore,Lock


class T0Mon(TemplatedPage):
  """
  T0Mon main page
  This page is the request handler: every call (page/static resource/AJAX call) is maneged! 
  """
  error="<b>T0Mon Internal Error: Resource not found! </b><BR/> "
    
  def __init__(self, config={}):
    """
    create the Cache and the ServiceConfiguration (both singleton object)
    """
    TemplatedPage.__init__(self, config)    
    ##config object, contains locks,constants,...
    self.config=ServiceConfiguration(self.config._internal_name,self.config.dictionary_())
    ## cache object for table in the first page
    self.cache=Cache(self.config)
   
  @expose
  def index(self, **kwargs):
    return self.config.redirectToIndex()
    
  @expose
  def static(self, *args, **kwargs):
    """
    static requests: CSS, images, js, static-html pages...
    """
    PATH = os.path.abspath(os.path.dirname(__file__))
    file=PATH+'/static/'+'/'.join(args)
    if os.path.exists(file):
      return serve_file(file)
    else:
      return "Resource NOT found"    

    
  #@expose 
  #def temp(self, *args, **kwargs): 
    #""" NOT USED 
    #temporary files are automatically erased 
    #"""
    ############################################static content request  
    #if args[0]=='files':
      #try:
        #file=args[1]
        #return Utils.getCachedFile(file) 
      #except Exception,e:
        #print e
    #return "No resource found"
     
  @expose 
  def table(self, *args, **kwargs):
    """
    Table content generator
    """
    cherrypy.response.headers['Pragma']= "no-cache" # this line stop the browser from caching the content
    try:
      name=args[0]
      #find the class with the name of the call
      contentClass=getattr(TableContents,name)
      # create such class
      content=contentClass(self.config,kwargs)
      
      #check if there is such class in the cache
      cache=self.cache.getCachedContent(content)
      if cache!=None:
        content.finalize()
        return cache
      
      
      #otherwise run it and return the result
      content.start()
      content.join()
      res= str(content)
      return res
    except  Exception, e:
      print e
      return self.error
    	  
  @expose 
  def plot(self, *args, **kwargs):
    """
    Plot content generator
    """
    cherrypy.response.headers['Pragma']= "no-cache"
    try:
      name=args[0]
      contentClass=getattr(PlotContents,name)
      content=contentClass(self.config,kwargs)
      
      content.start()
      content.join()
      res= str(content)
      return res
    except  Exception, e:
      print e
      return self.error
    	  
  @expose
  def form(self, *args, **kwargs):
    """
    Form content generator
    """
    try:
      name=args[0]
      ## forms are just function
      form=getattr(Forms,name)
      return form(self.config,kwargs)
    except  Exception, e:
      return self.error
    
    @expose 
    def default(self, *args, **kwargs):
      return self.config.redirectToIndex()


############################################################################ Singleton(s) Configuration object + cache
class ServiceConfiguration(object):
  """
   This class holds the constants (DASUrl,PlotfairyUrl) and the semaphores (threadSemaphore and emailLock) for the whole system
   PATTERN: Singleton
   
   NB: all the *new* objects are being derived from 'object' in order to make pydoc works
  """
  indexPath='/static/pages/index.html'
  imagePath='/static/imgs/'
  cacheTimeoutMinutes=4
  fetchingSemaphore=None
  DASMaxConnections=20 # MAX number of threads connection to DAS in parallel  
  ContentMaxThreads=30 # MAX threads of Contets
  
  def __init__(self,name,confDict):
    self.name=name
     ##check configuration ----->> ServiceConfig
    res,log=Utils.configurationCheck(confDict)
    if not(res):
      # if the configuration is wrong, T0Mon will not start!
      raise Exception('Invalid Configuration: '+log)
    self.DASUrl=confDict['DAS']
    self.plotfairyUrl=confDict['plotfairy']
    self.emailAlert=confDict['emailAlert']
    self.emailLock=Lock()
    self.threadSemaphore=BoundedSemaphore(self.DASMaxConnections)
    self.contentSemaphore=BoundedSemaphore(self.ContentMaxThreads)
    # why are we locking the connections to plotfairy (web server)?
    # the reason is with multiple parallel connection *sometimes* we were having troubles connecting
    # may be because plotfairy was still in development. 
    # If this is a bottleneck more testing can be done and eventually the lock can be removed!
    self.plotfairyLock=Lock()
    
  def sendAlarm(self,alarm):
    """
    alarm is sent to the mail in the config file
    """
    self.emailLock.acquire()
    try:
     Utils.sendAlarm(self, alarm)
    finally:
      self.emailLock.release()
    
  def getBaseUrl(self):
    return "/"+self.name+'/'
  
  def getImageUrl(self):
    return self.getBaseUrl()+self.imagePath
  
  def redirectToIndex(self):
    #return " Hi Stephen, <BR/> I am sorry but there is not a valid example now..."
    return """<meta HTTP-EQUIV="REFRESH" content="0; url=%s"> """%("/"+self.name+self.indexPath)
  
#############################################################
class Cache(object):
  """
  This class is a cache of contents, every content (with parameters) can be stored here.
  Default cached content are the ones in tableToCache
  Pattern: Singleton, Observer for Contents
  """
  cache=[]
  updating=[]
  tableToCache=[ ('RunDetails',{'page':'1'}),#('RunDetails',{'page':'2'}),
                ('ExprJ',{}),('ExprMJ',{}),('RepkJ',{}),('RepkMJ',{}),('RecoJ',{}),('RecoMJ',{}),('AlcaJ',{}),('AlcaMJ',{}),
                ('ExpressStatTable',{}),('ExpressMergeStatTable',{}),('RepackStatTable',{}),('RepackMergeStatTable',{}),
                ('RecoStatTable',{}),('RecoMergeStatTable',{}),('AlcaStatTable',{}),('AlcaMergeStatTable',{}),('DQMStatTable',{})
                ] 
  
  def __init__(self,config):
    """
     create cache and the contents to be cached
    """
    self.config=config  
    self.cacheLock=Lock()
    for (table,params) in self.tableToCache:
      try:        
        contentClass=getattr(TableContents,table)
        content=contentClass(self.config,params)
        content.addObserver(self)
        # I put content in cache (even if not computed) because then
        # I update the cache with only with new values of the contents already saved   
        self.cache.append(content)
        self.updating.append(content)
        #print "CREATING:"+content.name +"  with params="+str(content.parameterDict)
        content.start()  
      except Exception, e:
        print e
        
        
  def update(self,content):
    """
     when there is an update I store the new result in the cache
     the cache updating is working ONLY for contents already present in the cache
    """
    #every operation of the cache needs to be locked!
    self.cacheLock.acquire()
    try:
      j=0
      ## remove the content from the updating list
      while j<len(self.updating) and self.updating[j]!=content: j+=1
      if j<len(self.updating) and self.updating[j]==content:  del self.updating[j]
      i=0
      #remove the old content from the cache
      while i<len(self.cache) and self.cache[i]!=content: i+=1
      if i<len(self.cache) and self.cache[i]==content: 
        del self.cache[i]
        ## add the new content to the cache
        self.cache.append(content)
    finally:
      self.cacheLock.release()
    
    
  def updateOldCache(self):
    """
    check if the cache has expired and if so, lunch the process for updating
    """
    #I assume if I am already updating something there is no need to check all the cache contents for expired 
    if len(self.updating)>0: return
    #every operation of the cache needs to be locked!
    if not self.cacheLock.acquire(False): return
    try:
      for cachedTable in self.cache:
        try:
          # I update only if I am not already updating
          if not cachedTable in self.updating:
            dtime= cachedTable.generationTime+ timedelta(minutes=self.config.cacheTimeoutMinutes)
            # and if the time since last update is bigger then cacheTimeoutMinutes
            if dtime< datetime.now():
              contentClass=getattr(TableContents,cachedTable.name)
              content=contentClass(self.config,cachedTable.parameterDict)
              self.updating.append(content)
              content.addObserver(self)
              content.start()  
              #print 'CACHE: Updating '+ str(cachedTable.name)+"  with params="+str(cachedTable.parameterDict)+" cache dim="+str(len(self.cache))
        except Exception, e:
        #print "NOT updating"
          print e
    finally:
      self.cacheLock.release()

  def getCachedContent(self,content):
    """
     return cache value for content
     if content is not in the cache return None 
    """
    res=None
    # need to use a hierarchy of 'try' to be able to run pydoc
    try:
      try:
        self.updateOldCache()
        self.cacheLock.acquire()
        for cachedTable in self.cache:
          if cachedTable==content:
            res=str(cachedTable)
      except Exception,e:
        pass
    finally:
      self.cacheLock.release()
      if res=="":res=None # this happens only when the cache has not computed contents [aka: the very beginning]
    return res 
