"""
content protototypes
almost HTML-free package [just the HTML error message are left here]
"""
from Columns import *
from types import *
from Parameters import *
import threading,Tables,Utils
from DASManager import DASManager,DASCall,DASException
import TableContents
from PlotLib import  EmptyPlotException,PlotfairyException


class Content(threading.Thread):
  """
This class represent the SuperClass of every dynamic content (plots, tables).

I need a double behaviour: dynamic (making contents for the cache --Observable Object)
                           static (answering back to the original function -- Thread Object).
                                        
Patterns: Observable, Comparable, Executable
  """ 
  htmlData=""
  resDict={}
  parameters=[]
  generationTime=None
  url=""
 
  def __init__(self,config,inputParams={}):
    ## there is a Semaphore blocking the Thread creation: this is made to avoid errors because too many threads are created
    ## the lock is released at the end of the run() method.
    ## REMEMBER: if you create a content and you don't run it, you still need to release the lock (call the 'finalize' method)!!!
    config.contentSemaphore.acquire()
    self.threadCriticCode=True
    threading.Thread.__init__(self) 
    self.name=self.__class__.__name__
    self._observers = []
    self.config=config
    parameterDict={}
    ## safe mode to read parameter is via parameterDict: all the keys have a value (a default one if not matched with the input)
    for param in self.parameters:
      param.checkOnDict(inputParams)
      parameterDict.update({param.name: param.value})
    self.parameterDict=parameterDict
     
      
  def __cmp__(self, content):
    """
    2 contents are the same one if they have same name and same parameterDict
    """
    try:
      if isinstance(content, Content):
       return abs(cmp(self.name,content.name)) + abs(cmp(self.parameterDict.values(),content.parameterDict.values()))      
    except:
      pass
    return -1   
    
    
  def buildUrls(self):
    """
     This function is overridden in any content and return a list of urls to be fetched
    """
    return [self.url,]
    
  def dataProcessing(self):
    """
     This function is overridden in any content and process the fetched data
    """
    return []
  
  def addObserver(self, observer):
   if not observer in self._observers:
      self._observers.append(observer)

  def run(self):
    """
     Executable method
    """
    try:
      modifiedUrls=self.buildUrls()
      #this avoid  some errors
      if type(modifiedUrls) == StringType: modifiedUrls=[modifiedUrls,]
      # making DASManager and calling it on the urls
      rawData=[]
      if len(modifiedUrls)>0:
        DasManager=DASManager(self.config)
        self.resDict=DasManager.calls(modifiedUrls)
        #process data
        rawData=self.dataProcessing()
      #call decorator method from Tables
      decoratorMethod=getattr(Tables,self.dataDecorator)
      self.htmlData=decoratorMethod(self,rawData)
    except DASException, dase:
      #this exception rises when DAS is not reachable: alarm sent
      if not (Utils.testDAS(self.config)):
        alarm="DAS Service:  <span style=\"color:red\"> OFFLINE </span><BR/>"
        self.config.sendAlarm(alarm)
        self.htmlData=alarm
      else:
        self.htmlData="<b>Connection Error or Wrong URL</b> <BR/>"
    #except TableBuildingException, tbe:
        #self.htmlData="<B>"+tbe.getMsg()+"</B><BR/> Reload the page to re-build this table.<BR/>"
    except EmptyPlotException, epe:
      #this exception rises when there is no enough data to make a plot
      self.htmlData=self.getSummary()+ "<B>Empty Plot</B>"  
    except PlotfairyException, pe:
      #this exception rises when plotfairy doesn't respond: alarm sent
      alarm= "<BR/><B>PlotFairy connection error</B>"
      self.config.sendAlarm(alarm)
      self.htmlData=self.getSummary()+ alarm   
    except Exception, e:
      #general exception
      print "Exception=="+str(e)
      self.htmlData="<b>Connection Error or Wrong URL</b> <BR/>"
    finally:
       self.finalize()
 
    self.generationTime=datetime.now()    
    #update obeservers
    for observer in self._observers:
      observer.update(self)
    

  def finalize(self):
    if self.threadCriticCode:
      self.config.contentSemaphore.release()
      self.threadCriticCode=False
    
  def __str__(self):
    return self.htmlData
  
  def getHtml(self):
    return self.htmlData
  
#######################################################################################3 TableBuildingException
#class TableBuildingException(Exception):
  #"""
  #This Exception can be raised during the table creation to display to the user a simple string
  #"""
  #msg=""
  #def __init__(self,msg="Error building the table"):
    #self.msg=msg
    #Exception.__init__(self)
  
  #def getMsg(self):
    #return self.msg  

#########################################################################################  SimpleTableContent
class SimpleTableContent(Content):
   """
    no parameters, basic processing
   """
   url=[] 
   colDecorators=[]
   title=""
   dataDecoratorModule='Tables'
   dataDecorator='SimpleTable'

   def dataProcessing(self):
    return self.resDict[self.url]
###########################################################################################  LongTableContent
class LongTableContent(SimpleTableContent):
  """
   Table with count query and multiple pages [with page selection combobox and arrows]
  """
  mainUrl=""
  countUrl=""
  tableStepSize=10
  dataDecorator='LongTable' 
  parameters=[PageParameter()]
 
  def buildUrls(self): 
    self.downLimit =0   
    self.upLimit   = self.tableStepSize
    self.downLimit =(self.parameterDict['page'] -1) * self.tableStepSize 
    self.upLimit=self.downLimit+self.tableStepSize
    ###make das url options
    DASoptions='' 
    if '?' not in self.mainUrl: ## ExpressAcquiredJobs and RepackAcquiredJobs have an url-hard-coded parameter
      DASoptions+='?' 
    DASoptions+='&amp;'.join(map(lambda k:k.getDasReq(),self.parameters))
    ##build count query
    self.newUrls=["",""]
    self.newUrls[1]=self.countUrl+DASoptions
    ##building data query
    self.newUrls[0]=self.mainUrl+DASoptions +"&amp;uplimit="+str(self.upLimit) +'&amp;downlimit='+str(self.downLimit)
    return self.newUrls
    

  def dataProcessing(self):
    tableData=self.resDict[self.newUrls[0]]
    self.tableSize=self.resDict[self.newUrls[1]][0][0]
    try:
      self.title=self.title%self.parameterDict['status']
    except:
      pass
    return tableData
############################################################################################ RunLongTableContent
class RunLongTableContent(LongTableContent):
  """
  Long Table with Run Parameter
  """
  parameters=[PageParameter(),RunParameter()]
  
################################################################################################################
################################################################################################################
################################################################################################################
############################################ main page tables ##################################################


class RunDetails(SimpleTableContent):
  """ RunDetails is the main table of the homepage: special behaviour
  """
  dataDecorator='RunDetailTable' 
  tableStepSize=10
  title="Runs: %s"
  mainQuery='runfilesdetails'
  countQuery='lastruns'
  parameters=[PageParameter()]
  ### this special table requires many other support queries
  groupQueries=['numexprcompleted','numexpressstreamers','numexpresstotalstreamers','numrepacktemp','numrepackcompleted',
  'numrepackmergedexported','numrecotemp','numrecocompleted','numrecomergedexported','numalcatemp',
  'numalcacompleted','numalcamergedexported','numstreamers','numexpresstotal','numrepackedtotal',
  'numrepackmerged','numrecotemptotal','numrecototal','numrecomerged','numalcatemptotal','numalcatotal','numalcamerged']
  colDecorators=[RunId(),sTime(),eTime(),process(),version(),hlt(),
            acq(),status(),numstreamers(),expressGraph(),repackGraph(),
          recoGraph(),alcaGraph(),plotLinks()]
  
  
    
  def buildUrls(self):
    ## fetch runs
    t= DASCall(self.config,self.countQuery)
    t.start()
    t.join()
    self.lastruns=t.res
    nRuns=len(self.lastruns)
    self.tableSize=nRuns
    self.title=self.title%nRuns
    #update limits
    if nRuns==0:
      return [] 
    self.downLimit =(self.parameterDict['page']-1) * self.tableStepSize 
    self.upLimit=self.downLimit+min(self.tableStepSize,nRuns-self.downLimit)
    
    ## get run_id from lower/higher Limit
    self.higherRun=self.lastruns[self.downLimit]['run']
    self.lowerRun=self.lastruns[self.upLimit-1]['run']
    ####### build parameters for the calls
    self.urls={}
    for rawUrl in self.groupQueries:
         completeUrl=rawUrl+'?lowerrunid='+str(self.lowerRun)+'&amp;higherrunid='+str(self.higherRun)
         self.urls.update({rawUrl:completeUrl})
    for i in range(self.downLimit,self.upLimit):
      run=self.lastruns[i]['run']
      completeUrl=self.mainQuery+'?run='+str(run)
      self.urls.update({self.mainQuery+'-'+str(run):completeUrl})
    return self.urls.values()
   
      
  def dataProcessing(self):  
    tableData=[] 
    runDataDict={}
    for i in range(self.downLimit,self.upLimit):
       run=self.lastruns[i]['run']
       runDataDict.update({run:self.resDict[self.mainQuery+'?run='+str(run)]})
       for attr in self.groupQueries:
         runDataDict[run][0].update({attr:self.getAttrByRun(self.resDict[self.urls[attr]],run)})

    tableData= map(lambda k:k[0],runDataDict.values())
    #order the data for run_id
    tableData.sort(key=lambda dict: dict['run_id'],reverse=True )
    return tableData
         
    
    
  def getAttrByRun(self,valueList, run):
     """
     find the matching dictionary for the run in valueList, if not in the list produce one
     """
     index=self.binary_search(valueList,run,'run_id')
     if index>=0:
        return valueList[index]
     #else: print "index error "+str(run)+" on list=="+str(valueList)
     return {'count':0,'run_id':run}
  
  
  def binary_search(self,a, x, attrName, lo=0, hi=None):
    """
    simple binary search adaption
    """
    if hi is None:
        hi = len(a)
    while lo < hi:
        mid = (lo+hi)//2
        midval = int(a[mid][attrName])
        if midval < x:
            lo = mid+1
        elif midval > x: 
            hi = mid
        else:
            return mid
    return -1

     
######################################################################################## Heartbeats
class Heartbeats(SimpleTableContent):
   """
   top table of the homepage
   """
   url="heartbeatswithalarms"
   dataDecorator="HeartbeatsTable"
   htmlId='heart'
   
   def dataProcessing(self):
     tableData=SimpleTableContent.dataProcessing(self)
     for i in range(len(tableData)):
       time =tableData[i].get('last_updated')
       try:
         last_updated=mktime(localtime())-time
         tableData[i]['last_updated']=last_updated
         ##I save  the time in seconds from Unix epoch
       except Exception, e:
         tableData[i]['last_updated']="time error"
         pass
     return tableData

########################################################################################### Statistics
class StatTable(SimpleTableContent):
  """special statistic table: they are used in homepage,run page and detail page"""
  title="Table"
  attrStatus="status"
  attrCount="count"
  url=""
  parameters=[RunParameter()]
  tableStatuses=["Acquired","Failed", "Complete",]
  dataDecorator="simpleStatTable"
  
  def buildUrls(self):
    suffix=""
    if self.parameterDict['run_id']!=None:
      suffix=self.parameterDict['run_id']
    self.newUrl= self.url%suffix
    return [self.newUrl,]
    
  def dataProcessing(self):
    return self.resDict[self.newUrl]

### Statistical tables!
   
class ExpressStatTable(StatTable):
  title="Express stats"
  url="statsexpress?run=%s"
  link="/T0Mon/static/pages/detail.html?table=ExprJByStatus&amp;run_id=%s&amp;status=%s"
  #tableStatuses=['New','Used','Abandoned','Failure','Success'] #OLD schema
  
class RepackStatTable(StatTable):
  title="Repack stats"
  url="statsrepack?run=%s"
  link="/T0Mon/static/pages/detail.html?table=RepkJByStatus&amp;run_id=%s&amp;status=%s"
  #tableStatuses=['New','Used','Abandoned','Failure','Success']#OLD schema
  
class ExpressMergeStatTable(StatTable):
  title="Express Merge stats"
  url="statsexpressmerge?run=%s"
  link="/T0Mon/static/pages/detail.html?table=ExprMJByStatus&amp;run_id=%s&amp;status=%s"
  
class RepackMergeStatTable(StatTable):
  title="Repack Merge stats"
  url="statsrepackmerge?run=%s"
  link="/T0Mon/static/pages/detail.html?table=RepkMJByStatus&amp;run_id=%s&amp;status=%s"
  
class RecoStatTable(StatTable):
  title="Reco stats"
  url="statsreco?run=%s"
  link="/T0Mon/static/pages/detail.html?table=RecoJByStatus&amp;run_id=%s&amp;status=%s"

class RecoMergeStatTable(StatTable):
  title="Reco Merge stats"
  url="statsrecomerge?run=%s"
  link="/T0Mon/static/pages/detail.html?table=RecoMJByStatus&amp;run_id=%s&amp;status=%s"
  
class AlcaStatTable(StatTable):
  title="AlcaSkim stats"
  url="statsalca?run=%s"
  link="/T0Mon/static/pages/detail.html?table=AlcaJByStatus&amp;run_id=%s&amp;status=%s"

class AlcaMergeStatTable(StatTable):
  title="AlcaSkim Merge stats"
  url="statsalcamerge?run=%s"
  link="/T0Mon/static/pages/detail.html?table=AlcaMJByStatus&amp;run_id=%s&amp;status=%s"
  
class DQMStatTable(StatTable):
  title="DQM Job stats"
  url="statsdqm?run=%s"
  link="/T0Mon/static/pages/detail.html?table=DQMJByStatus&amp;run_id=%s&amp;status=%s"   
################################################################################################# Jobs tables (homepage)
class ExprJ(LongTableContent):
  """ expressJobs table: homepage"""
  mainUrl='lastacquiredstreamerjobs?fileset=express'
  countUrl='lastacquiredstreamerjobs_count?fileset=express'
  title="Express Acquired Jobs"
  path='history.html?source=Express'
  colDecorators=[JobId(path),RunId(),defTime(),nstreamers()]
  

class ExprMJ(LongTableContent):
  """ expressMerge Jobs table: homepage"""
  mainUrl='expressmergelastjobs'
  countUrl='expressmergelastjobs_count'
  title="Express Merge Acquired Jobs"
  path='history.html?source=ExpressMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum()]
    
class RepkJ(LongTableContent):
  """ repack Jobs table: homepage"""
  mainUrl='lastacquiredstreamerjobs?fileset=repack'
  countUrl='lastacquiredstreamerjobs_count?fileset=repack'
  title="Repack Acquired Jobs"
  path='history.html?source=Repack'
  colDecorators=[JobId(path),RunId(),runTime(),nstreamers(),stream()]
  
class RepkMJ(LongTableContent):
  """ repack Merge Jobs table: homepage"""
  mainUrl='repackmergelastacquired'
  countUrl='repackmergelastacquired_count'
  title="Repack Merge Acquired Jobs"
  path='history.html?source=RepackMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  
class RecoJ(LongTableContent):
  """ reco Jobs table: homepage"""
  mainUrl='recolastacquired'
  countUrl='recolastacquired_count'
  title="Reconstruction Acquired Jobs"
  path='history.html?source=Reco'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  
class RecoMJ(LongTableContent):
  """ reco Merge Jobs table: homepage"""  
  mainUrl='recomergelastacquired'
  countUrl='recomergelastacquired_count'
  title="Reco Merge Acquired Jobs"
  path='history.html?source=RecoMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
    
class AlcaJ(LongTableContent):
  """ alca Jobs table: homepage"""
  mainUrl='alcalastacquired'
  countUrl='alcalastacquired_count'
  title="Alcaskim Acquired Jobs"
  path='history.html?source=Alca'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  
class AlcaMJ(LongTableContent):
  """ alca Merge Jobs table: homepage"""
  mainUrl='alcamergelastacquired'
  countUrl='alcamergelastacquired_count'
  title="AlcaSkim Merge Acquired Jobs"
  path='history.html?source=AlcaMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
    
################################################################################# detail tables

class ExprJByStatus(LongTableContent):
  """ Express Jobs by Status table: detail page"""
  parameters=[PageParameter(),RunParameter(),StatusParameter()]
  mainUrl='expressjobsbystatus'
  countUrl='expressjobsbystatus_count'
  title="Express %s Jobs"
  path='history.html?source=Express'
  defColDecorators=[JobId(path),RunId(),defTime(),nstreamers()] # columns for completed/failed jobs !!!!!!!!!!change me!!!! 
  runColDecorators=[JobId(path),RunId(),runTime(),nstreamers()] # columns for acquired jobs
  tableStepSize=100
  
  def buildUrls(self):
    ##bug fix: for acquired jobs I want to show the 'running time'
    ##         for all the others just the definition time. 
    ##         But the query is the same! (returning definition time)
    ##         so I use a different decorator for acquired jobs 
    self.colDecorators=self.defColDecorators
    if self.parameterDict['status'].lower()=='acquired':
       self.colDecorators=self.runColDecorators
    return super(ExprJByStatus,self).buildUrls() 
  
class ExprMJByStatus(ExprJByStatus):
  """ Express Merge Jobs by Status table: detail page"""
  mainUrl='expressmergejobsbystatus'
  countUrl='expressmergejobsbystatus_count'
  title="Express Merge %s Jobs"
  path='history.html?source=ExpressMerge'  
  defColDecorators=[JobId(path),RunId(),defTime(),nstreamers()]
  runColDecorators=[JobId(path),RunId(),runTime(),nstreamers()]
  
class RepkJByStatus(ExprJByStatus):
  """ Repack Jobs by Status table: detail page"""
  mainUrl='repackjobsbystatus'
  countUrl='repackjobsbystatus_count'
  title="Repack %s Jobs"
  path='history.html?source=Repack'
  defColDecorators=[JobId(path),RunId(),defTime(),nstreamers(),stream()] 
  runColDecorators=[JobId(path),RunId(),runTime(),nstreamers(),stream()] 
 
class RepkMJByStatus(ExprJByStatus):
  """ Repack Merge Jobs by Status table: detail page"""
  mainUrl='repackmergejobsbystatus'
  countUrl='repackmergejobsbystatus_count'
  title="Repack Merge %s Jobs"
  path='history.html?source=RepackMerge'
  defColDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  runColDecorators=[JobId(path),RunId(),runTime(),filesNum(),dataset()] 
  
class RecoJByStatus(RepkMJByStatus):
  """ Reco Jobs by Status table: detail page"""
  mainUrl='recojobsbystatus'
  countUrl='recojobsbystatus_count'
  path='history.html?source=Reco'
  title="Reconstruction %s Jobs"
  defColDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  runColDecorators=[JobId(path),RunId(),runTime(),filesNum(),dataset()] 
    
class RecoMJByStatus(RepkMJByStatus):
  """ Reco Merge Jobs by Status table: detail page"""
  mainUrl='recomergejobsbystatus'
  countUrl='recomergejobsbystatus_count'
  path='history.html?source=RecoMerge'
  title="Reco Merge %s Jobs"
  defColDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  runColDecorators=[JobId(path),RunId(),runTime(),filesNum(),dataset()] 
    
class AlcaJByStatus(RepkMJByStatus):
  """ Alca Jobs by Status table: detail page"""
  mainUrl='alcajobsbystatus'
  countUrl='alcajobsbystatus_count'
  path='history.html?source=Alca'
  title="Alcaskim %s Jobs"
  defColDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  runColDecorators=[JobId(path),RunId(),runTime(),filesNum(),dataset()] 
    
class AlcaMJByStatus(RepkMJByStatus):
  """ Alca Merge Jobs by Status table: detail page"""
  mainUrl='alcamergejobsbystatus'
  countUrl='alcamergejobsbystatus_count'
  path='history.html?source=AlcaMerge'
  title="AlcaSkim Merge %s Jobs"
  defColDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset()]
  runColDecorators=[JobId(path),RunId(),runTime(),filesNum(),dataset()] 
    
class DQMJByStatus(LongTableContent):
  """ DQM Jobs by Status table: detail page"""
  parameters=[PageParameter(),RunParameter(),StatusParameter()]
  mainUrl='dqmjobsbystatus'
  path='history.html?source='
  countUrl='dqmjobsbystatus_count'
  title="DQM %s Jobs"
  colDecorators=[DQMJobId(path),RunId(),defTime(),filesNum(),dataset(),fileset()]
  tableStepSize=100  
##########################################################################################
##########################################################################################
##########################################################################################
######################################################################### run page tables

############################################################configuration
class configuration(SimpleTableContent):
   """ superclass for configuration tables: run page"""
   parameters=[RunParameter()]
   
   def buildUrls(self):
     self.url= self.url%str(self.parameterDict['run_id'])
     return [self.url,]
       
class ExpressConfig(configuration):
  title="Express Configuration"
  url="express_config?run=%s&amp;stream="
  colDecorators=[stream(),splitInProgress(),procVersion(),confUrl(),globalTag(),AMConfUrl(),expVersion(),scenario()]

class RepackConfig(configuration):
  title="Repack Configuration"
  url="repack_config?run=%s&amp;stream="
  colDecorators=[stream(),procVersion(),repVersion()]


class GeneralConfig(configuration):
  """This special Config Table is made merging the results of 2 queries"""
  title="Reco/Alca Configuration" 
  urls=["alca_config?run=%s&amp;","reco_config?run=%s&amp;"]
  colDecorators=[dataset(),roReco(),roCmsswVersion(),roGlobalTag(),roConfUrl(),roProcVersion(),alAlcaskim(),alCmsswVersion(),alConfUrl(),alProcVersion(),scenario()]
  
  def buildUrls(self):
    newUrls=[]
    for i in range(0,len(self.urls)):
      newUrls.append(self.urls[i]%(str(self.parameterDict['run_id'])))
    self.urls= newUrls
    return newUrls
     
  def dataProcessing(self):
    firstData=self.resDict[self.urls[0]]
    secondData=self.resDict[self.urls[1]]
    tableData=[]
    try: 
      ## in case of data miss, the normal codes rise an exception. If we want to show an empty table just uncomment the following 2 lines
      #if firstData==None or secondData==None:
        #return []
      #if len(firstData)==0 or len(secondData)==0:
        #return []
      for fLine in firstData:
        for sLine in secondData:
          if fLine['primary_dataset']==sLine['primary_dataset']:
            newLine={}
            newLine.update(self.keyfilter(fLine,'al_'))
            newLine.update(self.keyfilter(sLine,'ro_'))
            newLine.update({'dataset':sLine['primary_dataset']})
            tableData.append(newLine)
      return sorted(tableData, key=lambda line: line['dataset'])
    except e:
      print 'Excaption:' +str(e)
      return []
    
  def keyfilter(self,dict,prefix):
    res={}
    for k,v in dict.iteritems():
       res.update({prefix+k:v})
    return res

######################################################################

class Streamers(RunLongTableContent):
  """ Streamers files table: run page"""
  mainUrl='runstreamers'
  countUrl='runstreamers_count'
  title="Streamers files"
  path='history.html?source=Streamer'
  colDecorators=[StreamerId(path),lumiId(),filesize(),events(),stream(),lfn()]
  ##BUG FIX: deleted exportable() column
  
class ExprTF(RunLongTableContent):
  """ Express Temporary files table: run page"""
  mainUrl='runtempexpressfiles'
  countUrl='runtempexpressfiles_count'
  title="Temporary Express files"
  path='history.html?source=Express'
  colDecorators=[FileId(path),filesize(),events(),lfn()]
  
class ExprF(RunLongTableContent):
  """ Express  files table: run page"""
  mainUrl='runexpressfiles'
  countUrl='runexpressfiles_count'
  title="Express Files"
  path='history.html?source=ExpressMerge'
  colDecorators=[FileId(path),filesize(),events(),lfn()]  
    
class RepkTF(RunLongTableContent):
  """ Repack Temporary files table: run page"""
  mainUrl='runtemprepackfiles'
  countUrl='runtemprepackfiles_count'
  title="Temporary Repack files"
  path='history.html?source=Repack'
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
  
class RepkF(RunLongTableContent):
  """ Repack files table: run page"""
  mainUrl='runrepackfiles'
  countUrl='runrepackfiles_count'
  title=" Repack files"
  path='history.html?source=RepackMerge'
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
  
class RecoTF(RunLongTableContent):
  """ Reco Temporary files table: run page"""
  mainUrl='runtemprecofiles'
  countUrl='runtemprecofiles_count'
  title="Temporary Reconstruction Files"
  path='history.html?source=Reco'
  colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]
  
class RecoF(RunLongTableContent):
  """ Reco files table: run page"""
  mainUrl='runrecofiles'
  countUrl='runrecofiles_count'
  title="Reconstruction Files"
  path='history.html?source=RecoMerge'
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
    
class AlcaTF(RunLongTableContent):
  """ Alca Temporary files table: run page"""
  mainUrl='runtempalcafiles'
  countUrl='runtempalcafiles_count'
  title="Temporary Alcaskim Files"
  path='history.html?source=Alca'
  colDecorators=[FileId(path),filesize(),events(),dataset(),skim(),lfn()]
  
class AlcaF(RunLongTableContent):
  """ Alca files table: run page"""
  mainUrl='runalcafiles'
  countUrl='runalcafiles_count'
  title="AlcaSkim Files"
  path='history.html?source=AlcaMerge'
  colDecorators=[FileId(path),filesize(),events(),dataset(),skim(),lfn()]

############################################################################# history.html tables

class History(Content):
  parameters=[SourceParameter(),ValueParameter()]
  url='allhistory'
  dataDecorator='TableLayers'
  
  def buildUrls(self):
    DASoptions='?'+'&amp;'.join(map(lambda k:k.getDasReq(),self.parameters))
    self.url=self.url+DASoptions
    return [self.url,]
  
  def dataProcessing(self):
    allData=self.resDict[self.url]
    codes={}
    #print "ALLDATA="+str(allData)
    for tabledata in allData:
      for raw in tabledata:
        if 'tablename' in raw:
           tableName=raw['tablename']
           tempData=[]
        else: tempData.append(raw)
      methodToCall = getattr(TableContents,'H'+tableName)
      tableContent=methodToCall(self.config,{})
      tableContent.finalize()
      if tableContent!=None:
        codes[tableName]= Tables.SimpleTable(tableContent,data=tempData)
    return codes
    
    
    
    
    
#### all the table starting with H* is a history.html page table
class HStreamerFiles(Streamers):
  title="Streamers"
##
##  every layer has 4 tables:
##  the name of every table is Express/Repack/Reco/Alca
##   jobs  ||   files   || mergeJobs  || mergeFiles
##
## the names may mislead you: because actually the files are Temporary e non Temporary ['merge' in this case is a wrong prefix]
## unfortunatly this conceptual error was made also in the query script DAS/Allhistory.html 
##
##############################
class HExpressJobs(ExprJ):
  title=" Express Jobs"
  path='history.html?source=Express'
  colDecorators=[JobId(path),RunId(),defTime(),nstreamers(),status()]

class HExpressFiles(ExprF):
  path='history.html?source=Express'
  title="Temporary Express Files"
  colDecorators=[FileId(path),filesize(),events(),lfn()]

class HExpressMergeJobs(ExprMJ):
  title=" Express Merge Jobs"
  path='history.html?source=ExpressMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),status()]


class HExpressMergeFiles(ExprTF):
  path='history.html?source=ExpressMerge'
  title=" Express Files"
  colDecorators=[FileId(path),filesize(),events(),lfn()]
  
#############################
class HRepackJobs(Content):
  title=" Repack Jobs"
  path='history.html?source=Repack'
  colDecorators=[JobId(path),RunId(),defTime(),nstreamers(),stream(),status()]
  
class HRepackFiles(Content):
  path='history.html?source=Repack'
  title="Temporary Repack Files"
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]


class HRepackMergeJobs(Content):
  title=" Repack Merge Jobs"
  path='history.html?source=RepackMerge'
  colDecorators=[JobId(path),RunId(),filesNum(),dataset(),status()]  
  
class HRepackMergeFiles(Content):
  path='history.html?source=RepackMerge'
  title="Repack Files"
  colDecorators=[FileId(path),filesize(),events(),dataset(),lfn()]
#############################
class HRecoJobs(Content):
  title=" Reco Jobs"
  path='history.html?source=Reco'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset(),status()]
  
class HRecoFiles(Content):
  path='history.html?source=Reco'
  title="Temporary Reco Files"
  colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]


class HRecoMergeJobs(Content):
  title=" Reco Merge Jobs"
  path='history.html?source=RecoMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),tier(),dataset(),status()]  
  
class HRecoMergeFiles(Content):
  path='history.html?source=RecoMerge'
  title=" Reco Files "
  colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]
#############################
class HAlcaJobs(Content):
  title=" Alca Jobs"
  path='history.html?source=Alca'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset(),status()]
  
class HAlcaFiles(Content):
  path='history.html?source=Alca'
  title="Temporary Alca Files"
  colDecorators=[FileId(path),filesize(),events(),skim(),dataset(),lfn()]


class HAlcaMergeJobs(Content):
  title=" Alca Merge Jobs"
  path='history.html?source=AlcaMerge'
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),skim(),dataset(),status()]  
  
class HAlcaMergeFiles(Content):
  path='history.html?source=AlcaMerge'
  title=" Alca Files"
  colDecorators=[FileId(path),filesize(),events(),skim(),dataset(),lfn()]


#class HDQMFiles(Content):
  #path='history.html?source=DQM'
  #title="DQM Files"
  #colDecorators=[FileId(path),filesize(),events(),tier(),dataset(),lfn()]
  
class HDQMJobs(Content):
  path='history.html?source=DQM'
  title="DQM Jobs"
  colDecorators=[JobId(path),RunId(),defTime(),filesNum(),dataset(),status()]
